"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = require("http");
const https_1 = require("https");
const fs_extra_1 = require("fs-extra");
const webium_1 = require("webium");
const SocketIO = require("socket.io");
const ipchannel_1 = require("ipchannel");
const init_1 = require("../../init");
const module_1 = require("../tools/internal/module");
const internal_1 = require("../tools/internal");
const color_1 = require("../tools/internal/color");
const repl_1 = require("../tools/internal/repl");
require("./load-message");
require("./life-cycle");
require("./load-rpc");
require("./load-schedule");
require("./hot-reload");
exports.router = null;
exports.http = null;
exports.ws = null;
const tryImport = module_1.createImport(require);
let sseContainer = null;
internal_1.define(app, "router", () => exports.router);
internal_1.define(app, "http", () => exports.http);
internal_1.define(app, "ws", () => exports.ws);
internal_1.define(app, "sse", () => sseContainer);
internal_1.define(app, "channel", () => exports.router ? ipchannel_1.default : null);
app.serve = async function serve(serverId) {
    if (serverId && !serverId.startsWith("web-server")) {
        return app.rpc.serve(serverId);
    }
    let { type, port, timeout, options } = app.config.server.http;
    let WS = app.config.server.websocket;
    sseContainer = {};
    exports.router = new webium_1.App({
        cookieSecret: app.config.session.secret,
        domain: app.config.server.hostname
    });
    switch (type) {
        case "http":
            exports.http = new http_1.Server(exports.router.listener);
            break;
        case "https":
            exports.http = https_1.createServer(options, exports.router.listener);
            break;
        case "http2":
            exports.http = require("http2").createSecureServer(options, exports.router.listener);
            break;
    }
    if (WS.enabled) {
        if (!WS.port)
            exports.ws = SocketIO(exports.http, WS.options);
        else
            exports.ws = SocketIO(WS.port, WS.options);
        exports.ws = internal_1.inspectAs(exports.ws, "[Sealed Object]");
    }
    await Promise.resolve().then(() => require("../handlers/https-redirector"));
    await Promise.resolve().then(() => require("../handlers/http-init"));
    await Promise.resolve().then(() => require("../handlers/http-static"));
    await Promise.resolve().then(() => require("../handlers/http-xml"));
    await Promise.resolve().then(() => require("../handlers/http-session"));
    let httpBootstrap = init_1.APP_PATH + "/bootstrap/http";
    module_1.moduleExists(httpBootstrap) && tryImport(httpBootstrap);
    if (WS.enabled) {
        let wsBootstrap = init_1.APP_PATH + "/bootstrap/websocket";
        module_1.moduleExists(wsBootstrap) && tryImport(wsBootstrap);
    }
    if (typeof exports.http["setTimeout"] == "function") {
        exports.http["setTimeout"](timeout);
    }
    return new Promise((resolve, reject) => {
        exports.http.on("error", (err) => {
            console.log(color_1.red `${err.toString()}`);
            if (err.message.includes("listen")) {
                process.exit(1);
            }
            else {
                reject(err);
            }
        }).listen(port, async () => {
            try {
                if (await fs_extra_1.pathExists(app.controllers.path)) {
                    await module_1.importDirectory(app.controllers.path);
                }
                let finish = async () => {
                    app.serverId = "web-server-" + ipchannel_1.default.pid;
                    await app.hooks.lifeCycle.startup.invoke();
                    if (typeof process.send == "function") {
                        process.send("ready");
                    }
                    else {
                        console.log(internal_1.serveTip("Web", app.serverId, internal_1.baseUrl()));
                    }
                    await repl_1.serve(app.serverId);
                    resolve();
                };
                if (ipchannel_1.default.pid) {
                    await finish();
                }
                else {
                    ipchannel_1.default.on("connect", finish);
                }
            }
            catch (err) {
                reject(err);
            }
        });
    });
};
//# sourceMappingURL=index.js.map