"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const error_1 = require("../tools/internal/error");
const get = require("lodash/get");
const functions_1 = require("../tools/functions");
process.on("SIGINT", async () => {
    try {
        await app.hooks.lifeCycle.shutdown.invoke();
        if (app.rpc.server) {
            let { services = [] } = app.config.server.rpc[app.serverId];
            for (let service of services) {
                if (typeof service.instance(app.local).destroy === "function") {
                    await service.instance(app.local).destroy();
                }
            }
        }
        process.exit();
    }
    catch (err) {
        process.exit(1);
    }
}).on("message", msg => {
    if (msg == "shutdown") {
        process.emit("SIGINT", "SIGINT");
    }
});
app.hooks.lifeCycle.startup.bind(async () => {
    await app.services.schedule.instance(app.local).resume();
});
app.hooks.lifeCycle.shutdown.bind(async () => {
    await app.services.schedule.instance(app.local).stop();
});
app.hooks.lifeCycle.shutdown.bind(async () => {
    if (app.http) {
        await new Promise(resolve => {
            let timer = setTimeout(resolve, 500);
            app.http.close(() => {
                clearTimeout(timer);
                resolve();
            });
        });
        for (let id in app.sse) {
            app.sse[id].close();
        }
        await functions_1.sleep(500);
    }
});
app.hooks.lifeCycle.shutdown.bind(async () => {
    if (app.ws) {
        await new Promise(resolve => {
            let timer = setTimeout(resolve, 500);
            app.ws.close(() => {
                clearTimeout(timer);
                resolve();
            });
        });
    }
});
app.hooks.lifeCycle.startup.bind(() => {
    if (!app.ws)
        return;
    app.message.subscribe(app.message.ws.name, (context) => {
        let { target, volatile, local, event, data } = context;
        let ws = volatile ? app.ws.volatile : app.ws;
        ws = local ? ws.local : ws;
        let nsp = context.nsp ? ws.of(context.nsp) : ws;
        target && (nsp = nsp.to(target));
        nsp.emit(event, ...data);
    });
});
app.hooks.lifeCycle.startup.bind(() => {
    if (!app.sse)
        return;
    app.message.subscribe(app.message.sse.name, (context) => {
        let { close, target, event, data } = context;
        let targets;
        if (target) {
            targets = { [target]: app.sse[target] };
        }
        else {
            targets = app.sse;
        }
        for (let id in targets) {
            let sse = app.sse[id];
            if (sse) {
                if (close) {
                    sse.close();
                }
                else if (event) {
                    sse.emit(event, data);
                }
                else {
                    sse.send(data);
                }
            }
        }
    });
});
app.hooks.lifeCycle.startup.bind(() => {
    app.message.subscribe(app.schedule.name, async (context) => {
        let [modname, method, data] = context;
        let module = get(global, modname, null);
        if (module) {
            try {
                await module.instance(app.local)[method](...(data || []));
            }
            catch (err) {
                error_1.tryLogError(err);
            }
        }
    });
});
//# sourceMappingURL=life-cycle.js.map