"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../tools/internal");
const color_1 = require("../tools/internal/color");
const repl_1 = require("../tools/internal/repl");
const moment = require("moment");
const tasks = {};
function ensureServerId(serverId) {
    if (!app.config.server.rpc[serverId]) {
        throw new Error("The serverId is invalid");
    }
}
app.rpc = {
    server: null,
    connections: internal_1.inspectAs({}, "[Sealed Object]"),
    async serve(serverId) {
        ensureServerId(serverId);
        let servers = app.config.server.rpc;
        let _a = servers[serverId], { services = [] } = _a, options = tslib_1.__rest(_a, ["services"]);
        let service = await app.services.serve(options);
        for (let mod of services) {
            service.register(mod);
        }
        app.rpc.server = service;
        app.serverId = serverId;
        await app.hooks.lifeCycle.startup.invoke();
        for (let service of services) {
            if (typeof service.instance(app.local).init === "function") {
                await service.instance(app.local).init();
            }
        }
        console.log(internal_1.serveTip("RPC", serverId, service.dsn));
        await app.rpc.connect(serverId);
        await app.rpc.connectDependencies(serverId);
        await repl_1.serve(app.serverId);
    },
    async connect(serverId, defer = false) {
        ensureServerId(serverId);
        if (app.rpc.hasConnect(serverId))
            return;
        try {
            let servers = app.config.server.rpc;
            let _a = servers[serverId], { services } = _a, options = tslib_1.__rest(_a, ["services"]);
            let service = await app.services.connect(Object.assign({}, options, { id: app.serverId }));
            app.rpc.connections[serverId] = service;
            for (let mod of services) {
                service.register(mod);
                if (serverId !== app.serverId && mod === app.services.schedule) {
                    await app.services.schedule.instance(app.local).stop(true);
                }
            }
            if (tasks[serverId]) {
                app.schedule.cancel(tasks[serverId]);
                delete tasks[serverId];
            }
            app.message.linkRpcChannel(service);
            console.log(color_1.green `RPC server [${serverId}] connected.`);
        }
        catch (err) {
            if (defer) {
                if (!tasks[serverId]) {
                    tasks[serverId] = app.schedule.create({
                        salt: `connect-${serverId}`,
                        start: moment().unix(),
                        repeat: 1,
                    }, () => {
                        app.rpc.connect(serverId, defer);
                    });
                }
            }
            else {
                throw err;
            }
        }
    },
    async connectAll(defer = false) {
        let servers = app.config.server.rpc;
        let connections = [];
        for (let serverId in servers) {
            if (serverId !== app.serverId) {
                connections.push(app.rpc.connect(serverId, defer));
            }
        }
        await Promise.all(connections);
    },
    async connectDependencies(serverId) {
        serverId = serverId || app.serverId;
        let servers = app.config.server.rpc;
        let dependencies = servers[serverId]
            ? servers[serverId].dependencies
            : null;
        if (dependencies) {
            if (dependencies === "all") {
                await app.rpc.connectAll(true);
            }
            else {
                let metServers = [];
                for (let dependency of dependencies) {
                    for (let id in servers) {
                        if (id !== app.serverId && !metServers.includes(id)) {
                            if (servers[id].services.includes(dependency)) {
                                metServers.push(id);
                                await app.rpc.connect(id, true);
                            }
                        }
                    }
                }
            }
        }
    },
    hasConnect(serverId) {
        return !!app.rpc.connections[serverId];
    }
};
//# sourceMappingURL=load-rpc.js.map