"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const get = require("lodash/get");
const init_1 = require("../../init");
const Controller_1 = require("./Controller");
const StatusException_1 = require("../tools/StatusException");
const upload_1 = require("../tools/upload");
class HttpController extends Controller_1.Controller {
    constructor(req, res) {
        super();
        this.gzip = true;
        this.jsonp = false;
        this.csrfProtection = false;
        this.cors = false;
        this.uploadOptions = Object.assign({}, upload_1.UploadOptions);
        this.req = req;
        this.res = res;
        this.lang = (req.query && req.query.lang)
            || (req.cookies && req.cookies.lang)
            || req.lang
            || app.config.lang;
    }
    getAbsFilename(filename) {
        if (!path.isAbsolute(filename))
            filename = path.resolve(init_1.SRC_PATH, "views", filename);
        return filename;
    }
    view(path, vars = {}) {
        path = this.getAbsFilename(path);
        if (!("i18n" in vars)) {
            vars.i18n = (text, ...replacements) => {
                return this.i18n(text, ...replacements);
            };
        }
        try {
            let view = get(global, app.views.resolve(path));
            this.res.type = "text/html";
            return view.instance(path).render(vars);
        }
        catch (err) {
            if (err instanceof TypeError)
                throw new StatusException_1.StatusException(404);
            else
                throw err;
        }
    }
    send(data) {
        return this.res.send(data);
    }
    get session() {
        return this.req.session;
    }
    get url() {
        return this.req.url;
    }
    get sse() {
        return this.res.sse;
    }
    get isEventSource() {
        return this.req.isEventSource;
    }
    get csrfToken() {
        return this.req.csrfToken;
    }
    static httpErrorView(err, instance) {
        return instance.view(String(err.code), { err });
    }
}
exports.HttpController = HttpController;
//# sourceMappingURL=HttpController.js.map