"use strict";
var Hook_1;
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = require("path");
const chokidar_1 = require("chokidar");
const js_magic_1 = require("js-magic");
const function_intercepter_1 = require("function-intercepter");
const module_1 = require("./internal/module");
const tryImport = module_1.createImport(require);
let Hook = Hook_1 = class Hook {
    constructor(name, path) {
        this.name = name;
        this.children = {};
        this.path = path_1.normalize(path);
    }
    bind(handler) {
        let path = module_1.traceModulePath(this.path) || "<internal>";
        if (!Hook_1.Container[path]) {
            Hook_1.Container[path] = {};
        }
        if (!Hook_1.Container[path][this.name]) {
            Hook_1.Container[path][this.name] = [];
        }
        Hook_1.Container[path][this.name].push(handler);
        return this;
    }
    async invoke(input, output) {
        let result;
        for (let handler of this.getHandlers()) {
            let res = await handler(input, output);
            res === undefined || (result = res);
        }
        return result === undefined ? output : result;
    }
    decorate() {
        return function_intercepter_1.interceptAsync().before(async (...args) => {
            for (let handler of this.getHandlers()) {
                await handler(...args);
            }
        });
    }
    getHandlers() {
        let handlers = [];
        for (let container of Object.values(Hook_1.Container)) {
            let _handlers = container[this.name];
            if (_handlers && _handlers.length > 0) {
                handlers.push(..._handlers);
            }
        }
        return handlers;
    }
    __get(prop) {
        if (prop in this) {
            return this[prop];
        }
        else if (prop in this.children) {
            return this.children[prop];
        }
        else if (typeof prop != "symbol") {
            return this.children[prop] = new Hook_1(this.name + "." + String(prop), this.path);
        }
    }
    __has(prop) {
        return (prop in this) || (prop in this.children);
    }
};
Hook = Hook_1 = tslib_1.__decorate([
    js_magic_1.applyMagic,
    tslib_1.__metadata("design:paramtypes", [String, String])
], Hook);
exports.Hook = Hook;
(function (Hook) {
    Hook.Container = {};
})(Hook = exports.Hook || (exports.Hook = {}));
exports.Hook = Hook;
class HookProxy extends Hook {
    watch() {
        return chokidar_1.watch(this.path, {
            followSymlinks: false,
            awaitWriteFinish: true,
            ignored: /\.(js\.map|d\.ts|md)$/,
        }).on("add", filename => {
            tryImport(filename);
        }).on("change", filename => {
            this.clearCache(filename);
            tryImport(filename);
        }).on("unlink", this.clearCache.bind(this));
    }
    clearCache(filename) {
        let path = filename.slice(0, -path_1.extname(filename).length);
        path && (Hook.Container[path] = {});
        delete require.cache[filename];
    }
}
exports.HookProxy = HookProxy;
//# sourceMappingURL=Hook.js.map