"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const random = require("lodash/random");
const function_intercepter_1 = require("function-intercepter");
const HttpController_1 = require("../controllers/HttpController");
const StatusException_1 = require("./StatusException");
const RouteMap_1 = require("./RouteMap");
const module_1 = require("./internal/module");
var color_1 = require("./internal/color");
exports.green = color_1.green;
exports.grey = color_1.grey;
exports.red = color_1.red;
exports.yellow = color_1.yellow;
var error_1 = require("./internal/error");
exports.tryLogError = error_1.tryLogError;
var index_1 = require("./internal/index");
exports.isOwnMethod = index_1.isOwnMethod;
exports.isSubClassOf = index_1.isSubClassOf;
exports.ensureType = index_1.ensureType;
exports.define = index_1.define;
var module_2 = require("./internal/module");
exports.moduleExists = module_2.moduleExists;
exports.createImport = module_2.createImport;
var repl_1 = require("./internal/repl");
exports.serveRepl = repl_1.serve;
exports.connectRepl = repl_1.connect;
function sleep(timeout) {
    return new Promise(resolve => setTimeout(resolve, timeout));
}
exports.sleep = sleep;
function randStr(length = 5, chars = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ") {
    var str = "", max = chars.length - 1;
    for (let i = 0; i < length; i++) {
        str += chars[random(0, max)];
    }
    return str;
}
exports.randStr = randStr;
function injectCsrfToken(html, token) {
    var ele = `<input type="hidden" name="x-csrf-token" value="${token}">`, matches = html.match(/<form\s+.*?>/g);
    if (matches) {
        for (let match of matches) {
            let i = html.indexOf(match) + match.length, j = html.indexOf("<", i), spaces = html.substring(i, j);
            html = html.substring(0, i) + spaces + ele + html.substring(i);
        }
    }
    return html;
}
exports.injectCsrfToken = injectCsrfToken;
exports.requireAuth = function_intercepter_1.interceptAsync().before(function () {
    if (!this.authorized) {
        if (this instanceof HttpController_1.HttpController) {
            if (typeof this.fallbackTo === "string") {
                this.res.redirect(this.fallbackTo, 302);
            }
            else {
                this.res.send(this.fallbackTo);
            }
            return function_intercepter_1.intercept.BREAK;
        }
        else {
            throw new StatusException_1.StatusException(401);
        }
    }
});
let router, handle, wsTryImport;
function event(name) {
    return (proto, prop) => {
        let modPath = module_1.traceModulePath(app.controllers.path);
        if (!modPath)
            return;
        let { nsp = "/" } = proto.ctor;
        let data = {
            prefix: nsp,
            route: name,
            name: app.controllers.resolve(modPath),
        };
        let key = RouteMap_1.eventMap.keyFor(data);
        RouteMap_1.eventMap.add(key, prop);
        if (!wsTryImport)
            wsTryImport = require("../handlers/websocket-event").tryImport;
        if (!RouteMap_1.eventMap.isLocked(key)) {
            RouteMap_1.eventMap.lock(key);
            wsTryImport(nsp);
        }
    };
}
exports.event = event;
function route(method, path) {
    return (proto, prop) => {
        let modPath = module_1.traceModulePath(app.controllers.path);
        let { baseURI = "" } = proto.ctor;
        if (!modPath)
            return;
        if (!path) {
            let parts = method.split(/\s+/);
            method = parts[0] === "SSE" ? "GET" : parts[0];
            path = parts[1];
        }
        else if (method === "SSE") {
            method = "GET";
        }
        path = baseURI + path;
        let data = {
            prefix: method,
            route: path,
            name: app.controllers.resolve(modPath),
        };
        let key = RouteMap_1.routeMap.keyFor(data);
        RouteMap_1.routeMap.add(key, prop);
        if (!router || !handle) {
            router = require("../bootstrap/index").router;
            handle = require("../handlers/http-route").getRouteHandler;
        }
        if (!RouteMap_1.routeMap.isLocked(key)) {
            RouteMap_1.routeMap.lock(key);
            router.method(method, path, handle(key));
        }
    };
}
exports.route = route;
route.delete = function (path) {
    return route("DELETE", path);
};
route.get = function (path) {
    return route("GET", path);
};
route.head = function (path) {
    return route("HEAD", path);
};
route.patch = function (path) {
    return route("PATCH", path);
};
route.post = function (path) {
    return route("POST", path);
};
route.put = function (path) {
    return route("PUT", path);
};
route.sse = function (path) {
    return route("SSE", path);
};
//# sourceMappingURL=functions.js.map