"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const callSiteRecord = require("callsite-record");
const startsWith = require("lodash/startsWith");
const init_1 = require("../../../init");
const StatusException_1 = require("../StatusException");
const color_1 = require("./color");
const StackLine = /\((.+?):(\d+):(\d+)\)$|at\s+(.+?):(\d+):(\d+)$/;
function findAbsoluteFilename(stack, baseDir) {
    let matches = StackLine.exec(stack);
    if (matches) {
        let filename = (matches[1] || matches[4]).replace(init_1.SRC_PATH, init_1.APP_PATH);
        if (path.isAbsolute(filename) &&
            (!baseDir || startsWith(filename, baseDir))) {
            return filename;
        }
    }
    return void 0;
}
;
function resolveErrorStack(stack, baseDir, offset = 0) {
    let lines = stack.split("\n");
    let header = lines.splice(0, 1);
    let filename;
    let index = 0;
    for (let i = 0; i < lines.length; i++) {
        if (filename = findAbsoluteFilename(lines[i], baseDir)) {
            index = i;
            if (offset) {
                let _filename = findAbsoluteFilename(lines[index + offset]);
                if (_filename) {
                    index += offset;
                    filename = _filename;
                }
            }
            break;
        }
    }
    return {
        filename,
        stack: header.concat(lines.slice(index)).join("\n")
    };
}
exports.resolveErrorStack = resolveErrorStack;
async function tryLogError(err, stack) {
    if (err instanceof StatusException_1.StatusException && err.code < 500) {
        return;
    }
    else if (err instanceof Error) {
        err.stack = err.stack.replace(/default_\d\./g, "default.");
    }
    if (init_1.isDevMode && err instanceof Error) {
        let csr = callSiteRecord({
            forError: err,
        });
        if (csr) {
            try {
                let str = await csr.render({});
                console.log();
                console.log(err.toString());
                console.log();
                console.log(str);
                console.log();
            }
            catch (e) {
                console.log(err);
            }
        }
    }
    else {
        if (!(err instanceof Error)) {
            stack = err;
        }
        if (stack) {
            stack = `[${stack}]`;
        }
        else {
            stack = err.stack.split("\n")[1].trim();
        }
        console.log(color_1.red `${err.toString()} ${stack}`);
    }
}
exports.tryLogError = tryLogError;
//# sourceMappingURL=error.js.map