"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const color_1 = require("./color");
const truePattern = /^\s*(true|yes)\s*$/i;
const falsePattern = /^\s*(false|no)\s*$/i;
const nullPattern = /^\s*(null|nil|void)\s*$/i;
function isOwnMethod(obj, method) {
    return typeof obj[method] === "function" &&
        obj.constructor.prototype.hasOwnProperty(method);
}
exports.isOwnMethod = isOwnMethod;
function serveTip(type, serverId, url) {
    return color_1.green `${type} server [${serverId}](${url}) started.`;
}
exports.serveTip = serveTip;
function baseUrl() {
    let { server: { hostname, http: { port, type } } } = app.config, host = hostname + (port == 80 || port == 443 ? "" : ":" + port);
    return (type == "http2" ? "https" : type) + "://" + host;
}
exports.baseUrl = baseUrl;
function inspectAs(target, data) {
    target[util.inspect.custom] = () => data;
    return target;
}
exports.inspectAs = inspectAs;
function isSubClassOf(target, base) {
    return target.prototype instanceof base;
}
exports.isSubClassOf = isSubClassOf;
function define(target, prop, value) {
    let options = {
        enumerable: true,
        configurable: true,
    };
    if (typeof value === "function") {
        options.get = value;
    }
    else {
        options.value = value;
        options.writable = false;
    }
    return Object.defineProperty(target, prop, options);
}
exports.define = define;
function ensureType(value) {
    switch (typeof value) {
        case "string": {
            if (truePattern.test(value)) {
                return true;
            }
            else if (falsePattern.test(value)) {
                return false;
            }
            else if (nullPattern.test(value)) {
                return null;
            }
            else {
                let num = Number(value);
                if (!isNaN(num) && num <= Number.MAX_SAFE_INTEGER) {
                    return num;
                }
                else {
                    return value;
                }
            }
        }
        case "object": {
            if (Array.isArray(value)) {
                let arr = [];
                for (let item of value) {
                    arr.push(ensureType(item));
                }
                return arr;
            }
            else {
                let obj = {};
                for (let key in value) {
                    if (value.hasOwnProperty(key)) {
                        obj[key] = ensureType(value[key]);
                    }
                }
                return obj;
            }
        }
        default:
            return value;
    }
}
exports.ensureType = ensureType;
//# sourceMappingURL=index.js.map