"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-extra");
const FRON = require("fron");
const init_1 = require("../../../init");
const error_1 = require("./error");
const tryImport = createImport(require);
function moduleExists(name) {
    return fs.existsSync(name + (init_1.isTsNode ? ".ts" : ".js"));
}
exports.moduleExists = moduleExists;
function createImport(require) {
    return (id) => {
        try {
            let filename = require.resolve(id);
            if ([".json", ".jsonc"].includes(path.extname(filename))) {
                return FRON.parse(fs.readFileSync(filename, "utf8"));
            }
            else {
                return require(id);
            }
        }
        catch (err) {
            err.stack = error_1.resolveErrorStack(err.stack, null, 1).stack;
            error_1.tryLogError(err);
            return {};
        }
    };
}
exports.createImport = createImport;
function traceModulePath(baseDir) {
    let target = { stack: "" };
    Error.captureStackTrace(target);
    let { filename } = error_1.resolveErrorStack(target.stack, baseDir);
    let ext;
    if (!filename)
        return;
    if (ext = path.extname(filename)) {
        filename = filename.slice(0, -ext.length);
    }
    return filename;
}
exports.traceModulePath = traceModulePath;
async function importDirectory(dir) {
    var ext = init_1.isTsNode ? ".ts" : ".js";
    var files = await fs.readdir(dir);
    for (let file of files) {
        let filename = path.resolve(dir, file);
        let stat = await fs.stat(filename);
        if (stat.isFile() && path.extname(file) == ext) {
            tryImport(filename);
        }
        else if (stat.isDirectory()) {
            await importDirectory(filename);
        }
    }
}
exports.importDirectory = importDirectory;
//# sourceMappingURL=module.js.map